@@ -115,6 +115,8 @@ class FileSystemTree(object):
             file_id = file_info["id"]
             if file_name in parent[F.children]:
                 raise SameNameExistsError("same file name exists: %s" % file_name)
+            elif file_name == "":
+                raise InvalidValueError("file name can't be empty string: %s" % file_name)
             else:
                 parent[F.children][file_name] = {F.type: F.file, F.id: file_id}
                 self.files[file_id] = file_info            
@@ -325,9 +327,18 @@ class FileSystemTree(object):
                     }
                     if file_type == F.file:
                         file_id = file[F.children][name][F.id]
+                        file_info = self.files[file_id]
                         child["type"] = "file"
-                        child["size"] = self.files[file_id]["size"]
+                        child["size"] = file_info["size"]
                         child["id"] = file_id
+                        if "ctime" in file_info:
+                            child["ctime"] = file_info["ctime"]
+                        if "mtime" in file_info:
+                            child["mtime"] = file_info["mtime"]
+                        if "current_replica" in file_info:
+                            child["current_replica"] = file_info["current_replica"]
+                        if "replica" in file_info:
+                            child["replica"] = file_info["replica"]
                     elif file_type == F.dir:
                         child["type"] = "directory"
                         child["size"] = 0
